import 'dart:async';

import 'package:nexstate/src/nexstate_base.dart';

void main() {
  final userStore = UserStore();
  final subscription = userStore.runAndSubscribe(print);

  userStore.dispatch(
    const LogIn(
      User(
        firstName: 'Hamed',
        lastName: 'Aarab',
        userName: 'Hawmex',
      ),
    ),
  );

  userStore.dispatch(const LogOut());

  Timer(Duration.zero, subscription.cancel);
}

class User {
  final String firstName;
  final String lastName;
  final String userName;

  const User({
    required this.firstName,
    required this.lastName,
    required this.userName,
  });

  @override
  String toString() {
    return {
      'firstName': firstName,
      'lastName': lastName,
      'userName': userName,
    }.toString();
  }
}

class UserStore extends Store<User?> {
  UserStore() : super(null, logger: true);
}

class LogIn extends Action<User?> {
  final User user;

  const LogIn(this.user) : super();

  @override
  User activate(User? state) => user;
}

class LogOut extends Action<User?> {
  const LogOut() : super();

  @override
  // ignore: prefer_void_to_null
  Null activate(User? state) => null;
}
